@@ -1,7 +1,7 @@
 /*
  * self.c - example of a simple self monitoring task
  *
- * Copyright (C) 2002-2003 Hewlett-Packard Co
+ * Copyright (c) 2002-2006 Hewlett-Packard Development Company, L.P.
  * Contributed by Stephane Eranian <eranian@hpl.hp.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -22,7 +22,7 @@
  * OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  * This file is part of libpfm, a performance monitoring support library for
- * applications on Linux/ia64.
+ * applications on Linux.
  */
 
 #include <sys/types.h>
@@ -38,26 +38,24 @@
 #include <perfmon/pfmlib.h>
 #include <perfmon/perfmon.h>
 
-static char *event_list[]={
-	"cpu_cycles",
-	"IA64_INST_RETIRED",
-	NULL
-};
+#include "detect_pmcs.h"
 
 #define NUM_PMCS PFMLIB_MAX_PMCS
 #define NUM_PMDS PFMLIB_MAX_PMDS
 
-#define MAX_EVT_NAME_LEN	128
+#define TABSIZE 1024
+static int a[TABSIZE], b[TABSIZE];
 
 /*
  * our test code (function cannot be made static otherwise it is optimized away)
  */
-unsigned long
-noploop(unsigned long loop)
+uint64_t
+noploop(uint64_t loop)
 {
+	unsigned int j = 0;
 
-	while (loop--) {}
-	return loop;
+	while (loop--) { a[j] += a[j]*loop + b[j];j = (j+1)%TABSIZE; }
+	return a[0];
 }
 
 static void fatal_error(char *fmt,...) __attribute__((noreturn));
@@ -74,7 +72,6 @@ fatal_error(char *fmt, ...)
 	exit(1);
 }
 
-
 int
 main(int argc, char **argv)
 {
@@ -83,46 +80,44 @@ main(int argc, char **argv)
 	int ret, ctx_fd;
 	pfmlib_input_param_t inp;
 	pfmlib_output_param_t outp;
-	pfarg_reg_t pd[NUM_PMDS];
-	pfarg_reg_t pc[NUM_PMCS];
-	pfarg_context_t ctx[1];
+	pfarg_pmd_t pd[NUM_PMDS];
+	pfarg_pmc_t pc[NUM_PMCS];
+	pfarg_ctx_t ctx;
 	pfarg_load_t load_args;
 	pfmlib_options_t pfmlib_options;
 	unsigned int num_counters;
-	char name[MAX_EVT_NAME_LEN];
+	size_t len;
+	char *name;
 
 	/*
 	 * Initialize pfm library (required before we can use it)
 	 */
-	if (pfm_initialize() != PFMLIB_SUCCESS) {
-		printf("Can't initialize library\n");
-		exit(1);
-	}
+	ret = pfm_initialize();
+	if (ret != PFMLIB_SUCCESS)
+		fatal_error("Cannot initialize library: %s\n", pfm_strerror(ret));
+
+	pfm_get_max_event_name_len(&len);
+	name = malloc(len+1);
+	if (!name)
+		fatal_error("cannot allocate event name buffer\n");
+
 	pfm_get_num_counters(&num_counters);
-	
-	/*
-	 * check that the user did not specify too many events
-	 */
-	if ((unsigned int)(argc-1) > num_counters) {
-		printf("Too many events specified\n");
-		exit(1);
-	}
 
 	/*
 	 * pass options to library (optional)
 	 */
 	memset(&pfmlib_options, 0, sizeof(pfmlib_options));
-	pfmlib_options.pfm_debug = 0; /* set to 1 for debug */
+	pfmlib_options.pfm_debug   = 0; /* set to 1 for debug */
+	pfmlib_options.pfm_verbose = 1; /* set to 1 for verbose */
 	pfm_set_options(&pfmlib_options);
 
 	memset(pd, 0, sizeof(pd));
 	memset(pc, 0, sizeof(pc));
-	memset(ctx, 0, sizeof(ctx));
+	memset(&ctx, 0, sizeof(ctx));
 	memset(&load_args, 0, sizeof(load_args));
 
 	/*
-	 * prepare parameters to library. we don't use any Itanium
-	 * specific features here. so the pfp_model is NULL.
+	 * prepare parameters to library.
 	 */
 	memset(&inp,0, sizeof(inp));
 	memset(&outp,0, sizeof(outp));
@@ -130,11 +125,20 @@ main(int argc, char **argv)
 	/*
 	 * be nice to user!
 	 */
-	p = argc > 1 ? argv+1 : event_list;
-	for (i=0; *p ; i++, p++) {
-		if (pfm_find_event(*p, &inp.pfp_events[i].event) != PFMLIB_SUCCESS) {
-			fatal_error("Cannot find %s event\n", *p);
+	if (argc > 1) {
+		p = argv+1;
+		for (i=0; *p ; i++, p++) {
+			ret = pfm_find_full_event(*p, &inp.pfp_events[i]);
+			if (ret != PFMLIB_SUCCESS)
+				fatal_error("event %s: %s\n", *p, pfm_strerror(ret));
 		}
+	} else {
+		if (pfm_get_cycle_event(&inp.pfp_events[0]) != PFMLIB_SUCCESS)
+			fatal_error("cannot find cycle event\n");
+
+		if (pfm_get_inst_retired_event(&inp.pfp_events[1]) != PFMLIB_SUCCESS)
+			fatal_error("cannot find inst retired event\n");
+		i = 2;
 	}
 
 	/*
@@ -143,113 +147,114 @@ main(int argc, char **argv)
 	 */
 	inp.pfp_dfl_plm   = PFM_PLM3;
 
+	if (i > num_counters) {
+		i = num_counters;
+		printf("too many events provided (max=%d events), using first %d event(s)\n", num_counters, i);
+	}
+
 	/*
 	 * how many counters we use
 	 */
 	inp.pfp_event_count = i;
 
-	/*
-	 * let the library figure out the values for the PMCS
-	 */
-	if ((ret=pfm_dispatch_events(&inp, NULL, &outp, NULL)) != PFMLIB_SUCCESS) {
-		fatal_error("cannot configure events: %s\n", pfm_strerror(ret));
-	}
 	/*
 	 * now create a new context, per process context.
 	 * This just creates a new context with some initial state, it is not
 	 * active nor attached to any process.
 	 */
-	if (perfmonctl(0, PFM_CREATE_CONTEXT, ctx, 1) == -1 ) {
-		if (errno == ENOSYS) {
+	if (pfm_create_context(&ctx, NULL, 0) == -1 ) {
+		if (errno == ENOSYS)
 			fatal_error("Your kernel does not have performance monitoring support!\n");
-		}
 		fatal_error("Can't create PFM context %s\n", strerror(errno));
 	}
 
 	/*
 	 * extract the unique identifier for our context, a regular file descriptor
 	 */
-	ctx_fd = ctx[0].ctx_fd;
+	ctx_fd = ctx.ctx_fd;
+
+	/*
+	 * build the pfp_unavail_pmcs bitmask by looking
+	 * at what perfmon has available. It is not always
+	 * the case that all PMU registers are actually available
+	 * to applications. For instance, on IA-32 platforms, some
+	 * registers may be reserved for the NMI watchdog timer.
+	 *
+	 * With this bitmap, the library knows which registers NOT to
+	 * use. Of source, it is possible that no valid assignement may
+	 * be possible if certina PMU registers  are not available.
+	 */
+	detect_unavail_pmcs(ctx_fd, &inp.pfp_unavail_pmcs);
+
+	/*
+	 * let the library figure out the values for the PMCS
+	 */
+	if ((ret=pfm_dispatch_events(&inp, NULL, &outp, NULL)) != PFMLIB_SUCCESS)
+		fatal_error("cannot configure events: %s\n", pfm_strerror(ret));
 
 	/*
 	 * Now prepare the argument to initialize the PMDs and PMCS.
-	 * We must pfp_pmc_count to determine the number of PMC to intialize.
-	 * We must use pfp_event_count to determine the number of PMD to initialize.
-	 * Some events causes extra PMCs to be used, so  pfp_pmc_count may be >= pfp_event_count.
-	 *
-	 * This step is new compared to libpfm-2.x. It is necessary because the library no
-	 * longer knows about the kernel data structures.
+	 * We use pfp_pmc_count to determine the number of PMC to intialize.
+	 * We use pfp_pmd_count to determine the number of PMD to initialize.
+	 * Some events/features may cause extra PMCs to be used, leading to:
+	 * 	- pfp_pmc_count may be >= pfp_event_count
+	 * 	- pfp_pmd_count may be >= pfp_event_count
 	 */
-
 	for (i=0; i < outp.pfp_pmc_count; i++) {
 		pc[i].reg_num   = outp.pfp_pmcs[i].reg_num;
 		pc[i].reg_value = outp.pfp_pmcs[i].reg_value;
 	}
 
-	/*
-	 * the PMC controlling the event ALWAYS come first, that's why this loop
-	 * is safe even when extra PMC are needed to support a particular event.
-	 */
-	for (i=0; i < inp.pfp_event_count; i++) {
-		pd[i].reg_num   = pc[i].reg_num;
+	for (i=0; i < outp.pfp_pmd_count; i++) {
+		pd[i].reg_num   = outp.pfp_pmds[i].reg_num;
 	}
 
 	/*
 	 * Now program the registers
-	 *
-	 * We don't use the same variable to indicate the number of elements passed to
-	 * the kernel because, as we said earlier, pc may contain more elements than
-	 * the number of events (pmd) we specified, i.e., contains more than counting
-	 * monitors.
 	 */
-	if (perfmonctl(ctx_fd, PFM_WRITE_PMCS, pc, outp.pfp_pmc_count) == -1) {
-		fatal_error("perfmonctl error PFM_WRITE_PMCS errno %d\n",errno);
-	}
+	if (pfm_write_pmcs(ctx_fd, pc, outp.pfp_pmc_count))
+		fatal_error("pfm_write_pmcs error errno %d\n",errno);
 
-	if (perfmonctl(ctx_fd, PFM_WRITE_PMDS, pd, inp.pfp_event_count) == -1) {
-		fatal_error("perfmonctl error PFM_WRITE_PMDS errno %d\n",errno);
-	}
+	if (pfm_write_pmds(ctx_fd, pd, outp.pfp_pmd_count))
+		fatal_error("pfm_write_pmds error errno %d\n",errno);
 
 	/*
 	 * now we load (i.e., attach) the context to ourself
 	 */
 	load_args.load_pid = getpid();
-
-	if (perfmonctl(ctx_fd, PFM_LOAD_CONTEXT, &load_args, 1) == -1) {
-		fatal_error("perfmonctl error PFM_LOAD_CONTEXT errno %d\n",errno);
-	}
+	if (pfm_load_context(ctx_fd, &load_args))
+		fatal_error("pfm_load_context error errno %d\n",errno);
 
 	/*
 	 * Let's roll now
 	 */
-	pfm_self_start(ctx_fd);
+	if (pfm_start(ctx_fd, NULL))
+		fatal_error("pfm_start error errno %d\n",errno);
 
-	noploop(10000000);
+	noploop(1000000000ULL);
 
-	pfm_self_stop(ctx_fd);
+	if (pfm_stop(ctx_fd))
+		fatal_error("pfm_stop error errno %d\n",errno);
 
 	/*
-	 * now read the results
+	 * now read the results. We use pfp_event_count because
+	 * libpfm guarantees that counters for the events always
+	 * come first.
 	 */
-	if (perfmonctl(ctx_fd, PFM_READ_PMDS, pd, inp.pfp_event_count) == -1) {
-		fatal_error( "perfmonctl error READ_PMDS errno %d\n",errno);
-		return -1;
-	}
+	if (pfm_read_pmds(ctx_fd, pd, inp.pfp_event_count))
+		fatal_error( "pfm_read_pmds error errno %d\n",errno);
+
 	/*
 	 * print the results
-	 *
-	 * It is important to realize, that the first event we specified may not
-	 * be in PMD4. Not all events can be measured by any monitor. That's why
-	 * we need to use the pc[] array to figure out where event i was allocated.
-	 *
 	 */
 	for (i=0; i < inp.pfp_event_count; i++) {
-		pfm_get_event_name(inp.pfp_events[i].event, name, MAX_EVT_NAME_LEN);
-		printf("PMD%u %20"PRIu64" %s\n",
+		pfm_get_full_event_name(&inp.pfp_events[i], name, len+1);
+		printf("PMD%-3u %20"PRIu64" %s\n",
 			pd[i].reg_num,
 			pd[i].reg_value,
 			name);
 	}
+	free(name);
 	/*
 	 * and destroy our context
 	 */
