@@ -1,7 +1,7 @@
 /*
  * task.c - example of a task monitoring another one
  *
- * Copyright (C) 2002-2003 Hewlett-Packard Co
+ * Copyright (c) 2002-2006 Hewlett-Packard Development Company, L.P.
  * Contributed by Stephane Eranian <eranian@hpl.hp.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -20,9 +20,6 @@
  * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  * OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
- *
- * This file is part of libpfm, a performance monitoring support library for
- * applications on Linux/ia64.
  */
 #include <sys/types.h>
 #include <inttypes.h>
@@ -39,18 +36,13 @@
 #include <perfmon/pfmlib.h>
 #include <perfmon/perfmon.h>
 
+#include "detect_pmcs.h"
+
 #define NUM_PMCS PFMLIB_MAX_PMCS
 #define NUM_PMDS PFMLIB_MAX_PMDS
 
 #define MAX_EVT_NAME_LEN	128
 
-static char *event_list[]={
-	"cpu_cycles",
-	"IA64_INST_RETIRED",
-	NULL
-};
-
-
 static void fatal_error(char *fmt,...) __attribute__((noreturn));
 
 static void
@@ -87,14 +79,13 @@ child(char **arg)
 int
 parent(char **arg)
 {
-	char **p;
 	pfmlib_input_param_t inp;
 	pfmlib_output_param_t outp;
-	pfarg_context_t ctx[1];
-	pfarg_reg_t pc[NUM_PMCS];
-	pfarg_reg_t pd[NUM_PMDS];
+	pfarg_ctx_t ctx[1];
+	pfarg_pmc_t pc[NUM_PMCS];
+	pfarg_pmd_t pd[NUM_PMDS];
 	pfarg_load_t load_args;
-	unsigned int i;
+	unsigned int i, num_counters;
 	int status, ret;
 	int ctx_fd;
 	pid_t pid;
@@ -107,12 +98,14 @@ parent(char **arg)
 	memset(&outp,0, sizeof(outp));
 	memset(&load_args,0, sizeof(load_args));
 
-	p = event_list;
-	for (i=0; *p; i++, p++) {
-		if (pfm_find_event(*p, &inp.pfp_events[i].event) != PFMLIB_SUCCESS) {
-			fatal_error("Cannot find %s event\n", *p);
-		}
-	}
+	pfm_get_num_counters(&num_counters);
+
+	if (pfm_get_cycle_event(&inp.pfp_events[0]) != PFMLIB_SUCCESS)
+		fatal_error("cannot find cycle event\n");
+
+	if (pfm_get_inst_retired_event(&inp.pfp_events[1]) != PFMLIB_SUCCESS)
+		fatal_error("cannot find inst retired event\n");
+	i = 2;
 
 	/*
 	 * set the privilege mode:
@@ -124,18 +117,16 @@ parent(char **arg)
 	/*
 	 * how many counters we use
 	 */
+	if (num_counters < i) {
+		i = num_counters;
+		printf("too many events provided (max=%d events), using first %d event(s)\n", num_counters, i);
+	}
 	inp.pfp_event_count = i;
 
-	/*
-	 * let the library figure out the values for the PMCS
-	 */
-	if ((ret=pfm_dispatch_events(&inp, NULL, &outp, NULL)) != PFMLIB_SUCCESS) {
-		fatal_error("cannot configure events: %s\n", pfm_strerror(ret));
-	}
 	/*
 	 * now create a context. we will later attach it to the task we are creating.
 	 */
-	if (perfmonctl(0, PFM_CREATE_CONTEXT, ctx, 1) == -1) {
+	if (pfm_create_context(ctx, NULL, 0) == -1) {
 		if (errno == ENOSYS) {
 			fatal_error("Your kernel does not have performance monitoring support!\n");
 		}
@@ -147,42 +138,47 @@ parent(char **arg)
 	ctx_fd = ctx[0].ctx_fd;
 
 	/*
-	 * Now prepare the argument to initialize the PMDs and PMCS.
-	 * We must pfp_pmc_count to determine the number of PMC to intialize.
-	 * We must use pfp_event_count to determine the number of PMD to initialize.
-	 * Some events causes extra PMCs to be used, so  pfp_pmc_count may be >= pfp_event_count.
+	 * build the pfp_unavail_pmcs bitmask by looking
+	 * at what perfmon has available. It is not always
+	 * the case that all PMU registers are actually available
+	 * to applications. For instance, on IA-32 platforms, some
+	 * registers may be reserved for the NMI watchdog timer.
 	 *
-	 * This step is new compared to libpfm-2.x. It is necessary because the library no
-	 * longer knows about the kernel data structures.
+	 * With this bitmap, the library knows which registers NOT to
+	 * use. Of source, it is possible that no valid assignement may
+	 * be possible if certina PMU registers  are not available.
+	 */
+	detect_unavail_pmcs(ctx_fd, &inp.pfp_unavail_pmcs);
+
+	/*
+	 * let the library figure out the values for the PMCS
+	 */
+	if ((ret=pfm_dispatch_events(&inp, NULL, &outp, NULL)) != PFMLIB_SUCCESS) {
+		fatal_error("cannot configure events: %s\n", pfm_strerror(ret));
+	}
+	/*
+	 * Now prepare the argument to initialize the PMDs and PMCS.
+	 * We use pfp_pmc_count to determine the number of PMC to intialize.
+	 * We use pfp_pmd_count to determine the number of PMD to initialize.
+	 * Some events/features may cause extra PMCs to be used, leading to:
+	 * 	- pfp_pmc_count may be >= pfp_event_count
+	 * 	- pfp_pmd_count may be >= pfp_event_count
 	 */
 	for (i=0; i < outp.pfp_pmc_count; i++) {
 		pc[i].reg_num   = outp.pfp_pmcs[i].reg_num;
 		pc[i].reg_value = outp.pfp_pmcs[i].reg_value;
 	}
-
-	/*
-	 * the PMC controlling the event ALWAYS come first, that's why this loop
-	 * is safe even when extra PMC are needed to support a particular event.
-	 */
-	for (i=0; i < inp.pfp_event_count; i++) {
-		pd[i].reg_num   = pc[i].reg_num;
-	}
+	for(i=0; i < outp.pfp_pmd_count; i++)
+		pd[i].reg_num = outp.pfp_pmds[i].reg_num;
 
 	/*
 	 * Now program the registers
-	 *
-	 * We don't use the save variable to indicate the number of elements passed to
-	 * the kernel because, as we said earlier, pc may contain more elements than
-	 * the number of events we specified, i.e., contains more thann counting monitors.
 	 */
+	if (pfm_write_pmcs(ctx_fd, pc, outp.pfp_pmc_count) == -1)
+		fatal_error("pfm_write_pmcs error errno %d\n",errno);
 
-	if (perfmonctl(ctx_fd, PFM_WRITE_PMCS, pc, outp.pfp_pmc_count) == -1) {
-		fatal_error("perfmonctl error PFM_WRITE_PMCS errno %d\n",errno);
-	}
-
-	if (perfmonctl(ctx_fd, PFM_WRITE_PMDS, pd, inp.pfp_event_count) == -1) {
-		fatal_error("perfmonctl error PFM_WRITE_PMDS errno %d\n",errno);
-	}
+	if (pfm_write_pmds(ctx_fd, pd, outp.pfp_pmd_count) == -1)
+		fatal_error("pfm_write_pmds error errno %d\n",errno);
 
 	/*
 	 * Create the child task
@@ -202,30 +198,26 @@ parent(char **arg)
 	/*
 	 * check if process exited early
 	 */
-	if (WIFEXITED(status)) {
+	if (WIFEXITED(status))
 		fatal_error("command %s exited too early with status %d\n", arg[0], WEXITSTATUS(status));
-	}
+
 	/*
 	 * the task is stopped at this point
 	 */
 	
-	
 	/*
 	 * now we load (i.e., attach) the context to ourself
 	 */
 	load_args.load_pid = pid;
-
-	if (perfmonctl(ctx_fd, PFM_LOAD_CONTEXT, &load_args, 1) == -1) {
-		fatal_error("perfmonctl error PFM_LOAD_CONTEXT errno %d\n",errno);
-	}
+	if (pfm_load_context(ctx_fd, &load_args) == -1)
+		fatal_error("pfm_load_context error errno %d\n",errno);
 
 	/*
 	 * activate monitoring. The task is still STOPPED at this point. Monitoring
 	 * will not take effect until the execution of the task is resumed.
 	 */
-	if (perfmonctl(ctx_fd, PFM_START, NULL, 0) == -1) {
-		fatal_error("perfmonctl error PFM_START errno %d\n",errno);
-	}
+	if (pfm_start(ctx_fd, NULL) == -1)
+		fatal_error("pfm_start error errno %d\n",errno);
 
 	/*
 	 * now resume execution of the task, effectively activating
@@ -250,10 +242,9 @@ parent(char **arg)
 	/*
 	 * now simply read the results.
 	 */
-	if (perfmonctl(ctx_fd, PFM_READ_PMDS, pd, inp.pfp_event_count) == -1) {
-		fatal_error("perfmonctl error READ_PMDS errno %d\n",errno);
-		return -1;
-	}
+	if (pfm_read_pmds(ctx_fd, pd, inp.pfp_event_count) == -1)
+		fatal_error("pfm_read_pmds error errno %d\n",errno);
+
 	/*
 	 * print the results
 	 *
@@ -263,8 +254,8 @@ parent(char **arg)
 	 *
 	 */
 	for (i=0; i < inp.pfp_event_count; i++) {
-		pfm_get_event_name(inp.pfp_events[i].event, name, MAX_EVT_NAME_LEN);
-		printf("PMD%u %20"PRIu64" %s\n",
+		pfm_get_full_event_name(&inp.pfp_events[i], name, MAX_EVT_NAME_LEN);
+		printf("PMD%-3u %20"PRIu64" %s\n",
 			pd[i].reg_num,
 			pd[i].reg_value,
 			name);
@@ -281,6 +272,7 @@ int
 main(int argc, char **argv)
 {
 	pfmlib_options_t pfmlib_options;
+	int ret;
 
 	if (argc < 2) {
 		fatal_error("You must specify a command to execute\n");
@@ -288,16 +280,16 @@ main(int argc, char **argv)
 	/*
 	 * Initialize pfm library (required before we can use it)
 	 */
-	if (pfm_initialize() != PFMLIB_SUCCESS) {
-		printf("Can't initialize library\n");
-		exit(1);
-	}
+	ret = pfm_initialize();
+	if (ret != PFMLIB_SUCCESS)
+		fatal_error("Cannot initialize library: %s\n", pfm_strerror(ret));
 
 	/*
 	 * pass options to library (optional)
 	 */
 	memset(&pfmlib_options, 0, sizeof(pfmlib_options));
 	pfmlib_options.pfm_debug = 0; /* set to 1 for debug */
+	pfmlib_options.pfm_verbose= 0; /* set to 1 for verbose */
 	pfm_set_options(&pfmlib_options);
 
 	return parent(argv+1);
