@@ -1,7 +1,7 @@
 /*
  * pfmlib_itanium.c : support for Itanium-family PMU
  *
- * Copyright (C) 2001-2003 Hewlett-Packard Co
+ * Copyright (c) 2001-2006 Hewlett-Packard Development Company, L.P.
  * Contributed by Stephane Eranian <eranian@hpl.hp.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -20,11 +20,7 @@
  * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  * OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
- *
- * This file is part of libpfm, a performance monitoring support library for
- * applications on Linux/ia64.
  */
-
 #include <sys/types.h>
 #include <ctype.h>
 #include <string.h>
@@ -64,17 +60,35 @@
 #define pmc_thres	pmc_ita_count_reg.pmc_thres
 #define pmc_ism		pmc_ita_count_reg.pmc_ism
 
+/*
+ * Description of the PMC register mappings use by
+ * this module (as reported in pfmlib_reg_t.reg_num):
+ *
+ * 0 -> PMC0
+ * 1 -> PMC1
+ * n -> PMCn
+ *
+ * The following are in the model specific rr_br[]:
+ * IBR0 -> 0
+ * IBR1 -> 1
+ * ...
+ * IBR7 -> 7
+ * DBR0 -> 0
+ * DBR1 -> 1
+ * ...
+ * DBR7 -> 7
+ *
+ * We do not use a mapping table, instead we make up the
+ * values on the fly given the base.
+ */
+#define PFMLIB_ITA_PMC_BASE 0
+
+
 static int
 pfm_ita_detect(void)
 {
 	int ret = PFMLIB_ERR_NOTSUPP;
 
-	/*
-	 * check that the core library supports enough registers
-	 */
-	if (PFMLIB_MAX_PMCS < PMU_ITA_NUM_PMCS) return ret;
-	if (PFMLIB_MAX_PMDS < PMU_ITA_NUM_PMDS) return ret;
-
 	/*
 	 * we support all chips (there is only one!) in the Itanium family
 	 */
@@ -86,30 +100,42 @@ pfm_ita_detect(void)
  * Part of the following code will eventually go into a perfmon library
  */
 static int
-valid_assign(unsigned int *as, unsigned int cnt)
+valid_assign(unsigned int *as, pfmlib_regmask_t *r_pmcs, unsigned int cnt)
 {
 	unsigned int i;
-	for(i=0; i < cnt; i++) if (as[i]==0) return PFMLIB_ERR_NOASSIGN;
+	for(i=0; i < cnt; i++) {
+		if (as[i]==0) return PFMLIB_ERR_NOASSIGN;
+		/*
+		 * take care of restricted PMC registers
+		 */
+		if (pfm_regmask_isset(r_pmcs, as[i]))
+			return PFMLIB_ERR_NOASSIGN;
+	}
 	return PFMLIB_SUCCESS;
 }
 
 /*
  * Automatically dispatch events to corresponding counters following constraints.
- * Upon return the pfmlib_pmc_t structure is ready to be submitted to kernel
  */
 static int
 pfm_ita_dispatch_counters(pfmlib_input_param_t *inp, pfmlib_ita_input_param_t *mod_in, pfmlib_output_param_t *outp)
 {
+#define	has_counter(e,b)	(itanium_pe[e].pme_counters & (1 << (b)) ? (b) : 0)
 	pfmlib_ita_input_param_t *param = mod_in;
 	pfm_ita_pmc_reg_t reg;
-	pfmlib_event_t *e = inp->pfp_events;
-	pfmlib_pmc_t *pc = outp->pfp_pmcs;
+	pfmlib_event_t *e;
+	pfmlib_reg_t *pc, *pd;
+	pfmlib_regmask_t *r_pmcs;
 	unsigned int i,j,k,l, m;
 	unsigned int max_l0, max_l1, max_l2, max_l3;
 	unsigned int assign[PMU_ITA_NUM_COUNTERS];
-	unsigned int cnt = inp->pfp_event_count;
+	unsigned int cnt;
 
-#define	has_counter(e,b)	(itanium_pe[e].pme_counters & (1 << (b)) ? (b) : 0)
+	e      = inp->pfp_events;
+	pc     = outp->pfp_pmcs;
+	pd     = outp->pfp_pmds;
+	cnt    = inp->pfp_event_count;
+	r_pmcs = &inp->pfp_unavail_pmcs;
 
 	if (PFMLIB_DEBUG()) {
 		for (m=0; m < cnt; m++) {
@@ -124,8 +150,7 @@ pfm_ita_dispatch_counters(pfmlib_input_param_t *inp, pfmlib_ita_input_param_t *m
 	max_l2 = PMU_ITA_FIRST_COUNTER + PMU_ITA_NUM_COUNTERS*(cnt>2);
 	max_l3 = PMU_ITA_FIRST_COUNTER + PMU_ITA_NUM_COUNTERS*(cnt>3);
 
-	if (PFMLIB_DEBUG())
-		printf("max_l0=%u max_l1=%u max_l2=%u max_l3=%u\n", max_l0, max_l1, max_l2, max_l3);
+	DPRINT(("max_l0=%u max_l1=%u max_l2=%u max_l3=%u\n", max_l0, max_l1, max_l2, max_l3));
 	/*
 	 *  This code needs fixing. It is not very pretty and
 	 *  won't handle more than 4 counters if more become
@@ -136,7 +161,7 @@ pfm_ita_dispatch_counters(pfmlib_input_param_t *inp, pfmlib_ita_input_param_t *m
 
 		assign[0]= has_counter(e[0].event,i);
 
-		if (max_l1 == PMU_ITA_FIRST_COUNTER && valid_assign(assign,cnt) == PFMLIB_SUCCESS) goto done;
+		if (max_l1 == PMU_ITA_FIRST_COUNTER && valid_assign(assign, r_pmcs, cnt) == PFMLIB_SUCCESS) goto done;
 
 		for (j=PMU_ITA_FIRST_COUNTER; j < max_l1; j++) {
 
@@ -144,7 +169,7 @@ pfm_ita_dispatch_counters(pfmlib_input_param_t *inp, pfmlib_ita_input_param_t *m
 
 			assign[1] = has_counter(e[1].event,j);
 
-			if (max_l2 == PMU_ITA_FIRST_COUNTER && valid_assign(assign,cnt) == PFMLIB_SUCCESS) goto done;
+			if (max_l2 == PMU_ITA_FIRST_COUNTER && valid_assign(assign, r_pmcs, cnt) == PFMLIB_SUCCESS) goto done;
 
 			for (k=PMU_ITA_FIRST_COUNTER; k < max_l2; k++) {
 
@@ -152,14 +177,14 @@ pfm_ita_dispatch_counters(pfmlib_input_param_t *inp, pfmlib_ita_input_param_t *m
 
 				assign[2] = has_counter(e[2].event,k);
 
-				if (max_l3 == PMU_ITA_FIRST_COUNTER && valid_assign(assign,cnt) == PFMLIB_SUCCESS) goto done;
+				if (max_l3 == PMU_ITA_FIRST_COUNTER && valid_assign(assign, r_pmcs, cnt) == PFMLIB_SUCCESS) goto done;
 				for (l=PMU_ITA_FIRST_COUNTER; l < max_l3; l++) {
 
 					if(l == i || l == j || l == k) continue;
 
 					assign[3] = has_counter(e[3].event,l);
 
-					if (valid_assign(assign,cnt) == PFMLIB_SUCCESS) goto done;
+					if (valid_assign(assign, r_pmcs, cnt) == PFMLIB_SUCCESS) goto done;
 				}
 			}
 		}
@@ -167,7 +192,6 @@ pfm_ita_dispatch_counters(pfmlib_input_param_t *inp, pfmlib_ita_input_param_t *m
 	/* we cannot satisfy the constraints */
 	return PFMLIB_ERR_NOASSIGN;
 done:
-
 	for (j=0; j < cnt ; j++ ) {
 		reg.pmc_val = 0; /* clear all */
 		/* if plm is 0, then assume not specified per-event and use default */
@@ -179,20 +203,28 @@ done:
 		reg.pmc_umask  = is_ear(e[j].event) ? 0x0 : evt_umask(e[j].event);
 		reg.pmc_es     = itanium_pe[e[j].event].pme_code;
 
-		pc[j].reg_num   = assign[j];
-		pc[j].reg_value = reg.pmc_val;
+		pc[j].reg_num     = assign[j];
+		pc[j].reg_value   = reg.pmc_val;
+		pc[j].reg_addr    = assign[j];
 
-		__pfm_vbprintf("[pmc%u=0x%06lx thres=%d es=0x%02x plm=%d umask=0x%x pm=%d ism=0x%x oi=%d] %s\n",
-					assign[j], reg.pmc_val,
-					reg.pmc_thres,
-					reg.pmc_es,reg.pmc_plm,
-					reg.pmc_umask, reg.pmc_pm,
-					reg.pmc_ism,
-					reg.pmc_oi,
-					itanium_pe[e[j].event].pme_name);
+		pd[j].reg_num = assign[j];
+		pd[j].reg_addr = assign[j];
+
+		__pfm_vbprintf("[PMC%u(pmc%u)=0x%06lx thres=%d es=0x%02x plm=%d umask=0x%x pm=%d ism=0x%x oi=%d] %s\n",
+				assign[j],
+				assign[j],
+				reg.pmc_val,
+				reg.pmc_thres,
+				reg.pmc_es,reg.pmc_plm,
+				reg.pmc_umask, reg.pmc_pm,
+				reg.pmc_ism,
+				reg.pmc_oi,
+				itanium_pe[e[j].event].pme_name);
+		__pfm_vbprintf("[PMD%u(pmd%u)]\n", pd[j].reg_num, pd[j].reg_num);
 	}
 	/* number of PMC registers programmed */
 	outp->pfp_pmc_count = cnt;
+	outp->pfp_pmd_count = cnt;
 
 	return PFMLIB_SUCCESS;
 }
@@ -203,18 +235,21 @@ pfm_dispatch_iear(pfmlib_input_param_t *inp, pfmlib_ita_input_param_t *mod_in, p
 	pfm_ita_pmc_reg_t reg;
 	pfmlib_ita_input_param_t *param = mod_in;
 	pfmlib_ita_input_param_t fake_param;
-	pfmlib_pmc_t *pc = outp->pfp_pmcs;
-	int pos = outp->pfp_pmc_count;
+	pfmlib_reg_t *pc, *pd;
+	unsigned int pos1, pos2;
 	int iear_idx = -1;
 	unsigned int i, count;
 
+	pc = outp->pfp_pmcs;
+	pd = outp->pfp_pmds;
+	pos1 = outp->pfp_pmc_count;
+	pos2 = outp->pfp_pmd_count;
 	count = inp->pfp_event_count;
+
 	for (i=0; i < count; i++) {
 		if (is_iear(inp->pfp_events[i].event)) iear_idx = i;
 	}
 
-	if (iear_idx == 0) return PFMLIB_SUCCESS;
-
 	if (param == NULL || mod_in->pfp_ita_iear.ear_used == 0) {
 
 		/*
@@ -249,21 +284,30 @@ pfm_dispatch_iear(pfmlib_input_param_t *inp, pfmlib_ita_input_param_t *mod_in, p
 	reg.pmc10_ita_reg.iear_umask = param->pfp_ita_iear.ear_umask;
 	reg.pmc10_ita_reg.iear_ism   = param->pfp_ita_iear.ear_ism;
 
-	pc[pos].reg_num   = 10;  /* PMC10 is I-EAR config register */
-	pc[pos].reg_value = reg.pmc_val;
+	if (pfm_regmask_isset(&inp->pfp_unavail_pmcs, 10))
+		return PFMLIB_ERR_NOASSIGN;
 
-	pos++;
+	pc[pos1].reg_num     = 10;  /* PMC10 is I-EAR config register */
+	pc[pos1].reg_value   = reg.pmc_val;
+	pc[pos1].reg_addr    = 10;
+	pos1++;
+	pd[pos2].reg_num     = 0; 
+	pd[pos2++].reg_addr  = 0;
+	pd[pos2].reg_num     = 1; 
+	pd[pos2++].reg_addr  = 1;
 
-	__pfm_vbprintf("[pmc10=0x%lx tlb=%s plm=%d pm=%d ism=0x%x umask=0x%x]\n",
+	__pfm_vbprintf("[PMC10(pmc10)=0x%lx tlb=%s plm=%d pm=%d ism=0x%x umask=0x%x]\n",
 			reg.pmc_val,
 			reg.pmc10_ita_reg.iear_tlb ? "Yes" : "No",
 			reg.pmc10_ita_reg.iear_plm,
 			reg.pmc10_ita_reg.iear_pm,
 			reg.pmc10_ita_reg.iear_ism,
 			reg.pmc10_ita_reg.iear_umask);
+	__pfm_vbprintf("[PMD0(pmd0)]\n[PMD1(pmd1)\n");
 
 	/* update final number of entries used */
-	outp->pfp_pmc_count = pos;
+	outp->pfp_pmc_count = pos1;
+	outp->pfp_pmd_count = pos2;
 
 	return PFMLIB_SUCCESS;
 }
@@ -274,12 +318,17 @@ pfm_dispatch_dear(pfmlib_input_param_t *inp, pfmlib_ita_input_param_t *mod_in, p
 	pfm_ita_pmc_reg_t reg;
 	pfmlib_ita_input_param_t *param = mod_in;
 	pfmlib_ita_input_param_t fake_param;
-	pfmlib_pmc_t *pc = outp->pfp_pmcs;
-	int pos = outp->pfp_pmc_count;
+	pfmlib_reg_t *pc, *pd;
+	unsigned int pos1, pos2;
 	int dear_idx = -1;
 	unsigned int i, count;
 
+	pc = outp->pfp_pmcs;
+	pd = outp->pfp_pmds;
+	pos1 = outp->pfp_pmc_count;
+	pos2 = outp->pfp_pmd_count;
 	count = inp->pfp_event_count;
+
 	for (i=0; i < count; i++) {
 		if (is_dear(inp->pfp_events[i].event)) dear_idx = i;
 	}
@@ -320,12 +369,21 @@ pfm_dispatch_dear(pfmlib_input_param_t *inp, pfmlib_ita_input_param_t *mod_in, p
 	reg.pmc11_ita_reg.dear_umask = param->pfp_ita_dear.ear_umask;
 	reg.pmc11_ita_reg.dear_pt    = param->pfp_ita_drange.rr_used ? 0: 1;
 
-	pc[pos].reg_num    = 11;  /* PMC11 is D-EAR config register */
-	pc[pos].reg_value  = reg.pmc_val;
+	if (pfm_regmask_isset(&inp->pfp_unavail_pmcs, 11))
+		return PFMLIB_ERR_NOASSIGN;
 
-	pos++;
+	pc[pos1].reg_num     = 11;  /* PMC11 is D-EAR config register */
+	pc[pos1].reg_value   = reg.pmc_val;
+	pc[pos1].reg_addr    = 11;
+	pos1++;
+	pd[pos2].reg_num     = 2; 
+	pd[pos2++].reg_addr  = 2;
+	pd[pos2].reg_num     = 3; 
+	pd[pos2++].reg_addr  = 3;
+	pd[pos2].reg_num     = 17; 
+	pd[pos2++].reg_addr  = 17;
 
-	__pfm_vbprintf("[pmc11=0x%lx tlb=%s plm=%d pm=%d ism=0x%x umask=0x%x pt=%d]\n",
+	__pfm_vbprintf("[PMC11(pmc11)=0x%lx tlb=%s plm=%d pm=%d ism=0x%x umask=0x%x pt=%d]\n",
 			reg.pmc_val,
 			reg.pmc11_ita_reg.dear_tlb ? "Yes" : "No",
 			reg.pmc11_ita_reg.dear_plm,	
@@ -333,9 +391,11 @@ pfm_dispatch_dear(pfmlib_input_param_t *inp, pfmlib_ita_input_param_t *mod_in, p
 			reg.pmc11_ita_reg.dear_ism,
 			reg.pmc11_ita_reg.dear_umask,
 			reg.pmc11_ita_reg.dear_pt);
+	__pfm_vbprintf("[PMD2(pmd2)]\n[PMD3(pmd3)\nPMD17(pmd17)\n");
 
 	/* update final number of entries used */
-	outp->pfp_pmc_count = pos;
+	outp->pfp_pmc_count = pos1;
+	outp->pfp_pmd_count = pos2;
 
 	return PFMLIB_SUCCESS;
 }
@@ -345,7 +405,7 @@ pfm_dispatch_opcm(pfmlib_input_param_t *inp, pfmlib_ita_input_param_t *mod_in, p
 {
 	pfmlib_ita_input_param_t *param = mod_in;
 	pfm_ita_pmc_reg_t reg;
-	pfmlib_pmc_t *pc = outp->pfp_pmcs;
+	pfmlib_reg_t *pc = outp->pfp_pmcs;
 	int pos = outp->pfp_pmc_count;
 
 	if (param == NULL) return PFMLIB_SUCCESS;
@@ -354,11 +414,15 @@ pfm_dispatch_opcm(pfmlib_input_param_t *inp, pfmlib_ita_input_param_t *mod_in, p
 
 		reg.pmc_val = param->pfp_ita_pmc8.pmc_val;
 
+		if (pfm_regmask_isset(&inp->pfp_unavail_pmcs, 8))
+			return PFMLIB_ERR_NOASSIGN;
+
 		pc[pos].reg_num     = 8;
-		pc[pos++].reg_value = reg.pmc_val;
+		pc[pos].reg_value   = reg.pmc_val;
+		pc[pos++].reg_addr  = 8;
 
 
-		__pfm_vbprintf("[pmc8=0x%lx m=%d i=%d f=%d b=%d match=0x%x mask=0x%x]\n",
+		__pfm_vbprintf("[PMC8(pmc8)=0x%lx m=%d i=%d f=%d b=%d match=0x%x mask=0x%x]\n",
 				reg.pmc_val,
 				reg.pmc8_9_ita_reg.m,
 				reg.pmc8_9_ita_reg.i,
@@ -372,11 +436,16 @@ pfm_dispatch_opcm(pfmlib_input_param_t *inp, pfmlib_ita_input_param_t *mod_in, p
 
 		reg.pmc_val = param->pfp_ita_pmc9.pmc_val;
 
+		if (pfm_regmask_isset(&inp->pfp_unavail_pmcs, 9))
+			return PFMLIB_ERR_NOASSIGN;
+
 		pc[pos].reg_num     = 9;
-		pc[pos++].reg_value = reg.pmc_val;
+		pc[pos].reg_value   = reg.pmc_val;
+		pc[pos++].reg_addr  = 9;
 
 
-		__pfm_vbprintf("[pmc9 m=%d i=%d f=%d b=%d match=0x%x mask=0x%x]\n",
+		__pfm_vbprintf("[PMC9(pmc9)=0x%lx m=%d i=%d f=%d b=%d match=0x%x mask=0x%x]\n",
+				reg.pmc_val,
 				reg.pmc8_9_ita_reg.m,
 				reg.pmc8_9_ita_reg.i,
 				reg.pmc8_9_ita_reg.f,
@@ -395,13 +464,18 @@ pfm_dispatch_btb(pfmlib_input_param_t *inp, pfmlib_ita_input_param_t *mod_in, pf
 	pfm_ita_pmc_reg_t reg;
 	pfmlib_ita_input_param_t *param = mod_in;
 	pfmlib_ita_input_param_t fake_param;
-	pfmlib_pmc_t *pc = outp->pfp_pmcs;
+	pfmlib_reg_t *pc, *pd;
 	int found_btb=0;
 	unsigned int i, count;
-	int  pos = outp->pfp_pmc_count;
+	unsigned int  pos1, pos2;
 
 	reg.pmc_val = 0;
 
+	pc = outp->pfp_pmcs;
+	pd = outp->pfp_pmds;
+	pos1 = outp->pfp_pmc_count;
+	pos2 = outp->pfp_pmd_count;
+
 	count = inp->pfp_event_count;
 	for (i=0; i < count; i++) {
 		if (is_btb(inp->pfp_events[i].event)) found_btb = 1;
@@ -445,13 +519,15 @@ pfm_dispatch_btb(pfmlib_input_param_t *inp, pfmlib_ita_input_param_t *mod_in, pf
 	reg.pmc12_ita_reg.btbc_bpt = param->pfp_ita_btb.btb_tac & 0x1;
 	reg.pmc12_ita_reg.btbc_bac = param->pfp_ita_btb.btb_bac & 0x1;
 
-	memset(pc+pos, 0, sizeof(pfmlib_pmc_t));
+	if (pfm_regmask_isset(&inp->pfp_unavail_pmcs, 12))
+		return PFMLIB_ERR_NOASSIGN;
 
-	pc[pos].reg_num     = 12;
-	pc[pos++].reg_value = reg.pmc_val;
+	pc[pos1].reg_num   = 12;
+	pc[pos1].reg_value = reg.pmc_val;
+	pc[pos1].reg_value = 12;
+	pos1++;
 
-
-	__pfm_vbprintf("[pmc12=0x%lx plm=%d pm=%d tar=%d tm=%d ptm=%d ppm=%d bpt=%d bac=%d]\n",
+	__pfm_vbprintf("[PMC12(pmc12)=0x%lx plm=%d pm=%d tar=%d tm=%d ptm=%d ppm=%d bpt=%d bac=%d]\n",
 			reg.pmc_val,
 			reg.pmc12_ita_reg.btbc_plm,
 			reg.pmc12_ita_reg.btbc_pm,
@@ -462,8 +538,18 @@ pfm_dispatch_btb(pfmlib_input_param_t *inp, pfmlib_ita_input_param_t *mod_in, pf
 			reg.pmc12_ita_reg.btbc_bpt,
 			reg.pmc12_ita_reg.btbc_bac);
 
+	/*
+	 * PMD16 is included in list of used PMD
+	 */
+	for(i=8; i < 17; i++, pos2++) {
+		pd[pos2].reg_num = i;
+		pd[pos2].reg_addr = i;
+		__pfm_vbprintf("[PMD%u(pmd%u)]\n", pd[pos2].reg_num, pd[pos2].reg_num);
+	}
+
 	/* update final number of entries used */
-	outp->pfp_pmc_count = pos;
+	outp->pfp_pmc_count = pos1;
+	outp->pfp_pmd_count = pos2;
 
 	return PFMLIB_SUCCESS;
 }
@@ -494,7 +580,7 @@ check_intervals(pfmlib_ita_input_rr_t *irr, int mode, int *n_intervals)
 
 static void
 do_normal_rr(unsigned long start, unsigned long end,
-			     pfmlib_pmc_t *br, int nbr, int dir, int *idx, int *reg_idx, int plm)
+			     pfmlib_reg_t *br, int nbr, int dir, int *idx, int *reg_idx, int plm)
 {
 	unsigned long size, l_addr, c;
 	unsigned long l_offs = 0, r_offs = 0;
@@ -531,35 +617,31 @@ do_normal_rr(unsigned long start, unsigned long end,
 	l_size = l_addr - start;
 	r_size = end - l_addr-(1UL<<p2);
 
-	if (PFMLIB_DEBUG()) {
-		printf("largest chunk: 2^%d=0x%lx @0x%016lx-0x%016lx\n", p2, 1UL<<p2, l_addr, l_addr+(1UL<<p2));
-		if (l_size) printf("before: 0x%016lx-0x%016lx\n", start, l_addr);
-		if (r_size) printf("after : 0x%016lx-0x%016lx\n", l_addr+(1UL<<p2), end);
-	}
+	DPRINT(("largest chunk: 2^%d=0x%lx @0x%016lx-0x%016lx\n", p2, 1UL<<p2, l_addr, l_addr+(1UL<<p2)));
+	if (l_size) DPRINT(("before: 0x%016lx-0x%016lx\n", start, l_addr));
+	if (r_size) DPRINT(("after : 0x%016lx-0x%016lx\n", l_addr+(1UL<<p2), end));
 
 	if (dir == 0 && l_size != 0 && nbr == 1) {
 		p2++;
 		l_addr = end - (1UL << p2);
 		if (PFMLIB_DEBUG()) {
 			l_offs = start - l_addr;
-			printf(">>l_offs: 0x%lx\n", l_offs);
+			DPRINT((">>l_offs: 0x%lx\n", l_offs));
 		}
 	} else if (dir == 1 && r_size != 0 && nbr == 1) {
 		p2++;
 		l_addr = start;
 		if (PFMLIB_DEBUG()) {
 			r_offs = l_addr+(1UL<<p2) - end;
-			printf(">>r_offs: 0x%lx\n", r_offs);
+			DPRINT((">>r_offs: 0x%lx\n", r_offs));
 		}
 	}
 	l_size = l_addr - start;
 	r_size = end - l_addr-(1UL<<p2);
 	
-	if (PFMLIB_DEBUG()) {
-		printf(">>largest chunk: 2^%d @0x%016lx-0x%016lx\n", p2, l_addr, l_addr+(1UL<<p2));
-		if (l_size && !l_offs) printf(">>before: 0x%016lx-0x%016lx\n", start, l_addr);
-		if (r_size && !r_offs) printf(">>after : 0x%016lx-0x%016lx\n", l_addr+(1UL<<p2), end);
-	}
+	DPRINT((">>largest chunk: 2^%d @0x%016lx-0x%016lx\n", p2, l_addr, l_addr+(1UL<<p2)));
+	if (l_size && !l_offs) DPRINT((">>before: 0x%016lx-0x%016lx\n", start, l_addr));
+	if (r_size && !r_offs) DPRINT((">>after : 0x%016lx-0x%016lx\n", l_addr+(1UL<<p2), end));
 
 	/*
 	 * we initialize the mask to full 0 and
@@ -580,9 +662,11 @@ do_normal_rr(unsigned long start, unsigned long end,
 
 	br[*idx].reg_num     = *reg_idx;
 	br[*idx].reg_value   = l_addr;
+	br[*idx].reg_addr    = *reg_idx;
 
 	br[*idx+1].reg_num   = *reg_idx+1;
 	br[*idx+1].reg_value = db.val;
+	br[*idx+1].reg_addr  = *reg_idx+1;
 
 	*idx     += 2;
 	*reg_idx += 2;
@@ -613,26 +697,27 @@ do_normal_rr(unsigned long start, unsigned long end,
 
 
 static void
-print_one_range(pfmlib_ita_input_rr_desc_t *in_rr, pfmlib_ita_output_rr_desc_t *out_rr, pfmlib_pmc_t *dbr, int base_idx, int n_pairs)
+print_one_range(pfmlib_ita_input_rr_desc_t *in_rr, pfmlib_ita_output_rr_desc_t *out_rr, pfmlib_reg_t *dbr, int base_idx, int n_pairs)
 {
 	int j;
-	dbreg_t *d;
+	dbreg_t d;
 	unsigned long r_end;
 
-	printf("[0x%lx-0x%lx): %d register pair(s)\n", in_rr->rr_start, in_rr->rr_end, n_pairs);
-	printf("start offset: -0x%lx end_offset: +0x%lx\n", out_rr->rr_soff, out_rr->rr_eoff);
+	__pfm_vbprintf("[0x%lx-0x%lx): %d register pair(s)\n", in_rr->rr_start, in_rr->rr_end, n_pairs);
+	__pfm_vbprintf("start offset: -0x%lx end_offset: +0x%lx\n", out_rr->rr_soff, out_rr->rr_eoff);
 
 	for (j=0; j < n_pairs; j++, base_idx += 2) {
 
-		d     = (dbreg_t *)&dbr[base_idx+1].reg_value;
-		r_end = dbr[base_idx].reg_value+((~(d->db.db_mask)) & ~(0xffUL << 56));
+		d.val = dbr[base_idx+1].reg_value;
+		r_end = dbr[base_idx].reg_value+((~(d.db.db_mask)) & ~(0xffUL << 56));
 
-		printf("brp%u:  db%u: 0x%016lx db%u: plm=0x%x mask=0x%016lx end=0x%016lx\n",
+		__pfm_vbprintf("brp%u:  db%u: 0x%016lx db%u: plm=0x%x mask=0x%016lx end=0x%016lx\n",
 			dbr[base_idx].reg_num>>1,
 			dbr[base_idx].reg_num,
 			dbr[base_idx].reg_value,
 			dbr[base_idx+1].reg_num,
-			d->db.db_plm, d->db.db_mask,
+			d.db.db_plm,
+			(unsigned long) d.db.db_mask,
 			r_end);
 	}
 }
@@ -643,8 +728,8 @@ compute_normal_rr(pfmlib_ita_input_rr_t *irr, int dfl_plm, int n, int *base_idx,
 	pfmlib_ita_input_rr_desc_t *in_rr;
 	pfmlib_ita_output_rr_desc_t *out_rr;
 	unsigned long r_end;
-	pfmlib_pmc_t *br;
-	dbreg_t *d;
+	pfmlib_reg_t *br;
+	dbreg_t d;
 	int i, j, br_index, reg_idx, prev_index;
 
 	in_rr     = irr->rr_limits;
@@ -667,7 +752,7 @@ compute_normal_rr(pfmlib_ita_input_rr_t *irr, int dfl_plm, int n, int *base_idx,
 				4 - (reg_idx>>1), /* how many pairs available */
 				0,
 				&br_index,
-				&reg_idx, in_rr->rr_plm ? in_rr->rr_plm : (unsigned long)dfl_plm);
+				&reg_idx, in_rr->rr_plm ? in_rr->rr_plm : dfl_plm);
 
 		DPRINT(("br_index=%d reg_idx=%d\n", br_index, reg_idx));
 		/*
@@ -677,8 +762,8 @@ compute_normal_rr(pfmlib_ita_input_rr_t *irr, int dfl_plm, int n, int *base_idx,
 
 		for(j=prev_index; j < br_index; j+=2) {
 
-			d     = (dbreg_t *)&br[j+1].reg_value;
-			r_end = br[j].reg_value+((~(d->db.db_mask)+1) & ~(0xffUL << 56));
+			d.val = br[j+1].reg_value;
+			r_end = br[j].reg_value+((~(d.db.db_mask)+1) & ~(0xffUL << 56));
 
 			if (br[j].reg_value <= in_rr->rr_start)
 				out_rr->rr_soff = in_rr->rr_start - br[j].reg_value;
@@ -706,7 +791,7 @@ pfm_dispatch_irange(pfmlib_input_param_t *inp, pfmlib_ita_input_param_t *mod_in,
 {
 	pfm_ita_pmc_reg_t reg;
 	pfmlib_ita_input_param_t *param = mod_in;
-	pfmlib_pmc_t *pc = outp->pfp_pmcs;
+	pfmlib_reg_t *pc = outp->pfp_pmcs;
 	pfmlib_ita_input_rr_t *irr;
 	pfmlib_ita_output_rr_t *orr;
 	int pos = outp->pfp_pmc_count;
@@ -735,12 +820,14 @@ pfm_dispatch_irange(pfmlib_input_param_t *inp, pfmlib_ita_input_param_t *mod_in,
 
 	reg.pmc13_ita_reg.irange_ta = 0x0;
 
-	memset(pc+pos, 0, sizeof(pfmlib_pmc_t));
+	if (pfm_regmask_isset(&inp->pfp_unavail_pmcs, 13))
+		return PFMLIB_ERR_NOASSIGN;
 
 	pc[pos].reg_num     = 13;
-	pc[pos++].reg_value = reg.pmc_val;
+	pc[pos].reg_value   = reg.pmc_val;
+	pc[pos++].reg_addr  = 13;
 
-	__pfm_vbprintf("[pmc13=0x%lx ta=%d]\n", reg.pmc_val, reg.pmc13_ita_reg.irange_ta);
+	__pfm_vbprintf("[PMC13(pmc13)=0x%lx ta=%d]\n", reg.pmc_val, reg.pmc13_ita_reg.irange_ta);
 	
 	outp->pfp_pmc_count = pos;
 
@@ -752,7 +839,7 @@ pfm_dispatch_drange(pfmlib_input_param_t *inp, pfmlib_ita_input_param_t *mod_in,
 {
 	pfmlib_ita_input_param_t *param = mod_in;
 	pfmlib_event_t *e = inp->pfp_events;
-	pfmlib_pmc_t *pc = outp->pfp_pmcs;
+	pfmlib_reg_t *pc = outp->pfp_pmcs;
 	pfmlib_ita_input_rr_t *irr;
 	pfmlib_ita_output_rr_t *orr;
 	pfm_ita_pmc_reg_t reg;
@@ -784,14 +871,6 @@ pfm_dispatch_drange(pfmlib_input_param_t *inp, pfmlib_ita_input_param_t *mod_in,
 		if (is_dear(e[i].event)) return PFMLIB_SUCCESS; /* will be done there */
 	}
 
-	/*
-	 * this will clear the pt field which is what we want for drange
-	 * (reg.pmc11_ita_reg.dear_pt = 0)
-	 *
-	 * XXX: could even get rid of reg altogether here
-	 */
-	memset(pc+pos, 0, sizeof(pfmlib_pmc_t));
-
 	reg.pmc_val = 0UL;
 	/*
 	 * here we have no other choice but to use the default priv level as there is no
@@ -799,10 +878,14 @@ pfm_dispatch_drange(pfmlib_input_param_t *inp, pfmlib_ita_input_param_t *mod_in,
 	 */
 	reg.pmc11_ita_reg.dear_plm = inp->pfp_dfl_plm;
 
-	pc[pos].reg_num     = 11;
-	pc[pos++].reg_value = reg.pmc_val;
+	if (pfm_regmask_isset(&inp->pfp_unavail_pmcs, 11))
+		return PFMLIB_ERR_NOASSIGN;
 
-	__pfm_vbprintf("[pmc11=0x%lx tlb=%s plm=%d pm=%d ism=0x%x umask=0x%x pt=%d]\n",
+	pc[pos].reg_num     = 11;
+	pc[pos].reg_value   = reg.pmc_val;
+	pc[pos++].reg_addr  = 11;
+
+	__pfm_vbprintf("[PMC11(pmc11)=0x%lx tlb=%s plm=%d pm=%d ism=0x%x umask=0x%x pt=%d]\n",
 			reg.pmc_val,
 			reg.pmc11_ita_reg.dear_tlb ? "Yes" : "No",
 			reg.pmc11_ita_reg.dear_plm,	
@@ -991,68 +1074,17 @@ pfm_ita_get_ear_mode(unsigned int i, pfmlib_ita_ear_mode_t *m)
 }
 
 
-/*
- * Function used to print information about a specific events. More than
- * one event can be printed in case an event code is given rather than
- * a specific name. A callback function is used for printing.
- */
 static int
-pfm_ita_print_info(unsigned int v, int (*pf)(const char *fmt,...))
+pfm_ita_get_event_code(unsigned int i, unsigned int cnt, int *code)
 {
-	pme_ita_entry_t *e;
-        const char *quals[]={ "[Instruction Address Range]", "[OpCode Match]", "[Data Address Range]" };
-	long c;
-        int i;
+	if (cnt != PFMLIB_CNT_FIRST && (cnt < 4 || cnt > 7))
+		return PFMLIB_ERR_INVAL;
 
-	if (v >= PME_ITA_EVENT_COUNT || pf == NULL) return PFMLIB_ERR_INVAL;
-	e = itanium_pe+v;
+	*code = (int)itanium_pe[i].pme_code;
 
-	(*pf)("Umask  : ");
-	c = e->pme_umask;
-	for (i=3; i >=0; i--) {
-		(*pf)("%d", c & 1<<i ? 1 : 0);
-	}
-	(*pf)("\n");
-
-	(*pf)( "EAR    : %s (%s)\n",
-		e->pme_ear ? (e->pme_dear ? "Data" : "Inst") : "No",
-		e->pme_ear ? (e->pme_tlb ? "TLB Mode": "Cache Mode"): "N/A");
-	
-
-	(*pf)("BTB    : %s\n", e->pme_btb ? "Yes" : "No");
-
-	if (e->pme_maxincr > 1)
-		(*pf)("MaxIncr: %u  (Threshold [0-%u])\n", e->pme_maxincr,  e->pme_maxincr-1);
- 	else
-		(*pf)("MaxIncr: %u  (Threshold 0)\n", e->pme_maxincr);
-
-	(*pf)("Qual   : ");
-
-	c = e->pme_qualifiers.qual;
-	if ((c & 0x7) == 0) {
-		(*pf)("None");
-	} else {
-		for (i=0; i < 3; i++ ) {
-                	if (c & 0x1) (*pf)("%s ", quals[i]);
-                	c >>= 1;
-        	}
-	}
-	(*pf)("\n");
 	return PFMLIB_SUCCESS;
 }
 
-static int
-pfm_ita_get_event_code(unsigned int i)
-{
-	return itanium_pe[i].pme_code;
-}
-
-static unsigned long
-pfm_ita_get_event_vcode(unsigned int i)
-{
-	return itanium_pe[i].pme_entry_code.pme_vcode;
-}
-
 /*
  * This function is accessible directly to the user
  */
@@ -1073,23 +1105,18 @@ pfm_ita_get_event_name(unsigned int i)
 static void
 pfm_ita_get_event_counters(unsigned int j, pfmlib_regmask_t *counters)
 {
-	unsigned int i = 0;
+	unsigned int i;
 	unsigned long m;
 
 	memset(counters, 0, sizeof(*counters));
 
-	m =itanium_pe[i].pme_counters;
-	for(; m ; i++, m>>=1) {
-		if (m & 0x1) PFMLIB_REGMASK_SET(counters, i);
+	m =itanium_pe[j].pme_counters;
+	for(i=0; m ; i++, m>>=1) {
+		if (m & 0x1)
+			pfm_regmask_set(counters, i);
 	}
 }
 
-static void
-pfm_ita_num_counters(unsigned int *num)
-{
-	*num = PMU_ITA_NUM_COUNTERS;
-}
-
 static void
 pfm_ita_get_impl_pmcs(pfmlib_regmask_t *impl_pmcs)
 {
@@ -1098,7 +1125,8 @@ pfm_ita_get_impl_pmcs(pfmlib_regmask_t *impl_pmcs)
 	memset(impl_pmcs, 0, sizeof(*impl_pmcs));
 
 	/* all pmcs are contiguous */
-	for(i=0; i < PMU_ITA_NUM_PMCS; i++) PFMLIB_REGMASK_SET(impl_pmcs, i);
+	for(i=0; i < PMU_ITA_NUM_PMCS; i++)
+		pfm_regmask_set(impl_pmcs, i);
 }
 
 static void
@@ -1109,7 +1137,8 @@ pfm_ita_get_impl_pmds(pfmlib_regmask_t *impl_pmds)
 	memset(impl_pmds, 0, sizeof(*impl_pmds));
 
 	/* all pmds are contiguous */
-	for(i=0; i < PMU_ITA_NUM_PMDS; i++) PFMLIB_REGMASK_SET(impl_pmds, i);
+	for(i=0; i < PMU_ITA_NUM_PMDS; i++)
+		pfm_regmask_set(impl_pmds, i);
 }
 
 static void
@@ -1120,7 +1149,8 @@ pfm_ita_get_impl_counters(pfmlib_regmask_t *impl_counters)
 	memset(impl_counters, 0, sizeof(*impl_counters));
 
 	/* counting pmds are contiguous */
-	for(i=4; i < 8; i++) PFMLIB_REGMASK_SET(impl_counters, i);
+	for(i=4; i < 8; i++)
+		pfm_regmask_set(impl_counters, i);
 }
 	
 static void
@@ -1129,36 +1159,38 @@ pfm_ita_get_hw_counter_width(unsigned int *width)
 	*width = PMU_ITA_COUNTER_WIDTH;
 }
 
-static void
-pfm_ita_num_pmcs(unsigned int *width)
+static int
+pfm_ita_get_cycle_event(pfmlib_event_t *e)
 {
-	*width = PMU_ITA_NUM_PMCS;
+	e->event = PME_ITA_CPU_CYCLES;
+	return PFMLIB_SUCCESS;
+
 }
 
-static void
-pfm_ita_num_pmds(unsigned int *width)
+static int
+pfm_ita_get_inst_retired(pfmlib_event_t *e)
 {
-	*width = PMU_ITA_NUM_PMDS;
+	e->event = PME_ITA_IA64_INST_RETIRED;
+	return PFMLIB_SUCCESS;
 }
 
-
-
 pfm_pmu_support_t itanium_support={
 	.pmu_name		= "itanium",
 	.pmu_type		= PFMLIB_ITANIUM_PMU,
 	.pme_count		= PME_ITA_EVENT_COUNT,
+	.pmc_count		= PMU_ITA_NUM_PMCS,
+	.pmd_count		= PMU_ITA_NUM_PMDS,
+	.num_cnt		= PMU_ITA_NUM_COUNTERS,
 	.get_event_code		= pfm_ita_get_event_code,
-	.get_event_vcode	= pfm_ita_get_event_vcode,
 	.get_event_name		= pfm_ita_get_event_name,
 	.get_event_counters	= pfm_ita_get_event_counters,
-	.print_info		= pfm_ita_print_info,
 	.dispatch_events	= pfm_ita_dispatch_events,
-	.get_num_counters	= pfm_ita_num_counters,
-	.get_num_pmcs		= pfm_ita_num_pmcs,
-	.get_num_pmds		= pfm_ita_num_pmds,
 	.pmu_detect		= pfm_ita_detect,
 	.get_impl_pmcs		= pfm_ita_get_impl_pmcs,
 	.get_impl_pmds		= pfm_ita_get_impl_pmds,
 	.get_impl_counters	= pfm_ita_get_impl_counters,
-	.get_hw_counter_width	= pfm_ita_get_hw_counter_width
+	.get_hw_counter_width	= pfm_ita_get_hw_counter_width,
+	.get_cycle_event	= pfm_ita_get_cycle_event,
+	.get_inst_retired_event = pfm_ita_get_inst_retired
+	/* no event description available for Itanium */
 };
